(********************************************************************
 * COPYRIGHT -- Microsoft
 ********************************************************************
 * Library: SIKODrvLib
 * File: sikoAg2x.st
 * Author: Christian Hoffmann
 * Created: October 09, 2014
 ********************************************************************
 * Implementation OF library SIKODrvLib
 ********************************************************************) 

(* TODO: Add your comment here *)
FUNCTION_BLOCK fbSikoAg2x

	(* Control Word *)
	controlWord.0:= PdOutAg2x.ControlWordAg2x.bOff1;
	controlWord.1:= PdOutAg2x.ControlWordAg2x.bOff2;
	controlWord.2:= PdOutAg2x.ControlWordAg2x.bOff3;
	controlWord.3:= PdOutAg2x.ControlWordAg2x.bIntermediateStop;
	controlWord.4:= PdOutAg2x.ControlWordAg2x.bStartTravelJob;
	controlWord.5:= PdOutAg2x.ControlWordAg2x.bAcknowledgeError;
	controlWord.6:= PdOutAg2x.ControlWordAg2x.bInchingMode1;
	controlWord.7:= PdOutAg2x.ControlWordAg2x.bInching2Pos;
	controlWord.8:= PdOutAg2x.ControlWordAg2x.bInching2Neg;
	controlWord.9:= PdOutAg2x.ControlWordAg2x.bit9;
	controlWord.10:= PdOutAg2x.ControlWordAg2x.bMoveRelative;
	controlWord.11:= PdOutAg2x.ControlWordAg2x.bit11;
	controlWord.12:= PdOutAg2x.ControlWordAg2x.bit12;
	controlWord.13:= PdOutAg2x.ControlWordAg2x.bit13;
	controlWord.14:= PdOutAg2x.ControlWordAg2x.bit14;
	controlWord.15:= PdOutAg2x.ControlWordAg2x.bit15;
	
	(* Target Value *)
	targetValue:= PdOutAg2x.targetValue;
	
	(* Digital Outputs *)
	digitalOutputsControl.0:= PdOutAg2x.DoAg2x.do1;
	digitalOutputsControl.1:= PdOutAg2x.DoAg2x.bit1;
	digitalOutputsControl.2:= PdOutAg2x.DoAg2x.bit2;
	digitalOutputsControl.3:= PdOutAg2x.DoAg2x.bit3;
	digitalOutputsControl.4:= PdOutAg2x.DoAg2x.bit4;
	digitalOutputsControl.5:= PdOutAg2x.DoAg2x.bit5;
	digitalOutputsControl.6:= PdOutAg2x.DoAg2x.bit6;
	digitalOutputsControl.7:= PdOutAg2x.DoAg2x.bit7;

	(* Edge Detection *)
	fbR_TRIG1(CLK:= startRead);
	osrStartRead := fbR_TRIG1.Q;
	
	fbR_TRIG2(CLK:= startWrite);
	osrStartWrite := fbR_TRIG2.Q;
	
	fbR_TRIG3(CLK:= startCopy);
	osrStartCopy := fbR_TRIG3.Q;
	
	fbR_TRIG4(CLK:= startReadDataObj);
	osrStartReadDataObj:= fbR_TRIG4.Q;
	
	fbR_TRIG5(CLK:= startWriteDataObj);
	osrStartWriteDataObj:= fbR_TRIG5.Q;
	
	(* State Machines *)
	CASE rdState OF
		RD_STATE_WAIT_RTRIG: (* wait for rising edge at input startRead *)
			IF osrStartRead AND (wrState = 0) AND (copyState = 0) AND (rdDataObjState = 0) AND (wrDataObjState = 0) THEN
				IF (parameterValid = TRUE) THEN
					busy := 1;
					error := 0;
					done := 0;
					status := 16#FFFF; (* busy *)
					errorinfo := 0;
					rdState := 1;
					counter_read := 0;
					counterRead := 0;
				ELSE
					busy := 0;
					error := 1;
					done := 0;
					status := 50000; (* no data object loaded *)
					errorinfo := 0;
				END_IF;	
			END_IF;
		RD_STATE_INIT_EPL_SDO_READ: (* init EplSDORead *)
			counter_read := counter_read + 1;
			IF (counter_read > 93) THEN
				(* no more parameters to read *)
				counter_read := 0;
				counterRead := 0;
				rdState := RD_STATE_WAIT_RTRIG;
				busy := 0;
				error := 0;
				done := 1;
				status := 0; (* no error *)
				errorinfo := 0;
			ELSE
				(* init EplSDORead *)
				CASE counter_read OF
					1:  (* LED Functionality *)
						fbEplSDORead.index := 16#2201;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2201_LedFunctionality_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2201_LedFunctionality_r);
					2:  (* Service Interface Baudrate *)
						fbEplSDORead.index := 16#2221;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2221_ServiceInterfaceBaudrate_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2221_ServiceInterfaceBaudrate_r);
					3:  (* Digital Output 1 Functionality *)
						fbEplSDORead.index := 16#2301;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2301_Do1Functionality_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2301_Do1Functionality_r);
					4:  (* Digital Output Functionalities State *)
						fbEplSDORead.index := 16#2302;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2302_DoFunctionalitiesState_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2302_DoFunctionalitiesState_r);
					5:  (* Digital Outputs Polarity *)
						fbEplSDORead.index := 16#2303;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2303_DoPolarity_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2303_DoPolarity_r);
					6:  (* Digital Input 1 Functionality *)
						fbEplSDORead.index := 16#2401;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2401_Di1Functionality_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2401_Di1Functionality_r);
					7:  (* Digital Input 2 Functionality *)
						fbEplSDORead.index := 16#2402;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2402_Di2Functionality_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2402_Di2Functionality_r);
					8:  (* Digital Input 3 Functionality *)
						fbEplSDORead.index := 16#2403;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2403_Di3Functionality_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2403_Di3Functionality_r);
					9:  (* Digital Input 4 Functionality *)
						fbEplSDORead.index := 16#2404;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2404_Di4Functionality_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2404_Di4Functionality_r);
					10:  (* Digital Input Functionalities State *)
						fbEplSDORead.index := 16#2405;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2405_DiFunctionalitiesState_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2405_DiFunctionalitiesState_r);
					11:  (* Digital Inputs Polarity *)
						fbEplSDORead.index := 16#2406;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2406_DiPolarity_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2406_DiPolarity_r);
					12:  (* Controller Parameter P *)
						fbEplSDORead.index := 16#2601;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2601_ControllerParameterP_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2601_ControllerParameterP_r);
					13:  (* Controller Parameter I *)
						fbEplSDORead.index := 16#2602;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2602_ControllerParameterI_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2602_ControllerParameterI_r);
					14:  (* Controller Parameter D *)
						fbEplSDORead.index := 16#2603;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2603_ControllerParameterD_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2603_ControllerParameterD_r);
					15:  (* A-Pos *)
						fbEplSDORead.index := 16#2604;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2604_APos_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2604_APos_r);
					16:  (* V-Pos *)
						fbEplSDORead.index := 16#2605;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2605_VPos_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2605_VPos_r);
					17:  (* D-Pos *)
						fbEplSDORead.index := 16#2606;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2606_DPos_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2606_DPos_r);
					18:  (* A-Rot *)
						fbEplSDORead.index := 16#2607;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2607_ARot_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2607_ARot_r);
					19:  (* A-Inch *)
						fbEplSDORead.index := 16#2608;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2608_AInch_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2608_AInch_r);
					20:  (* V-Inch *)
						fbEplSDORead.index := 16#2609;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2609_VInch_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2609_VInch_r);
					21:  (* Pos Window *)
						fbEplSDORead.index := 16#260A;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n260A_PosWindow_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n260A_PosWindow_r);
					22:  (* Gear Ratio Numerator *)
						fbEplSDORead.index := 16#260B;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n260B_GearRatioNumerator_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n260B_GearRatioNumerator_r);
					23:  (* Gear Ratio Denominator *)
						fbEplSDORead.index := 16#260C;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n260C_GearRatioDenominator_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n260C_GearRatioDenominator_r);
					24:  (* Spindle Pitch *)
						fbEplSDORead.index := 16#260D;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n260D_SpindlePitch_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n260D_SpindlePitch_r);
					25:  (* Calibration Value *)
						fbEplSDORead.index := 16#260E;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n260E_CalibrationValue_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n260E_CalibrationValue_r);
					26:  (* Software Limit 1 *)
						fbEplSDORead.index := 16#260F;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n260F_SoftwareLimit1_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n260F_SoftwareLimit1_r);
					27:  (* Software Limit 2 *)
						fbEplSDORead.index := 16#2610;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2610_SoftwareLimit2_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2610_SoftwareLimit2_r);
					28:  (* Delta Inch *)
						fbEplSDORead.index := 16#2611;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2611_DeltaInch_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2611_DeltaInch_r);
					29:  (* Sense of Rotation *)
						fbEplSDORead.index := 16#2612;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2612_SenseOfRotation_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2612_SenseOfRotation_r);
					30:  (* Pos Type *)
						fbEplSDORead.index := 16#2613;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2613_PosType_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2613_PosType_r);
					31:  (* Operating Mode *)
						fbEplSDORead.index := 16#2614;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2614_OperatingMode_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2614_OperatingMode_r);
					32:  (* Inching 2 Stop Mode *)
						fbEplSDORead.index := 16#2615;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2615_Inching2StopMode_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2615_Inching2StopMode_r);
					33:  (* Inpos Mode *)
						fbEplSDORead.index := 16#2616;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2616_InposMode_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2616_InposMode_r);
					34:  (* Loop Length *)
						fbEplSDORead.index := 16#2617;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2617_LoopLength_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2617_LoopLength_r);
					35:  (* Contouring Error Limit *)
						fbEplSDORead.index := 16#2618;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2618_ContouringErrorLimit_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2618_ContouringErrorLimit_r);
					36:  (* Current Limiting *)
						fbEplSDORead.index := 16#2619;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2619_CurrentLimiting_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2619_CurrentLimiting_r);
					37:  (* Inching 2 Offset *)
						fbEplSDORead.index := 16#261A;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n261A_Inching2Offset_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n261A_Inching2Offset_r);
					38:  (* Inching 2 Acceleration Type *)
						fbEplSDORead.index := 16#261B;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n261B_Inching2AccelerationType_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n261B_Inching2AccelerationType_r);
					39:  (* Offset Value *)
						fbEplSDORead.index := 16#261C;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n261C_OffsetValue_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n261C_OffsetValue_r);
					40:  (* PCM Position 1 *)
						fbEplSDORead.index := 16#2922;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2922_PcmPosition1_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2922_PcmPosition1_r);
					41:  (* PCM Position 2 *)
						fbEplSDORead.index := 16#2923;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2923_PcmPosition2_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2923_PcmPosition2_r);
					42:  (* PCM Position 3 *)
						fbEplSDORead.index := 16#2924;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2924_PcmPosition3_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2924_PcmPosition3_r);
					43:  (* PCM Position 4 *)
						fbEplSDORead.index := 16#2925;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2925_PcmPosition4_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2925_PcmPosition4_r);
					44:  (* PCM Position 5 *)
						fbEplSDORead.index := 16#2926;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2926_PcmPosition5_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2926_PcmPosition5_r);
					45:  (* PCM Position 6 *)
						fbEplSDORead.index := 16#2927;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2927_PcmPosition6_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2927_PcmPosition6_r);
					46:  (* PCM Position 7 *)
						fbEplSDORead.index := 16#2928;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2928_PcmPosition7_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2928_PcmPosition7_r);
					47:  (* PCM Acceleration 1 *)
						fbEplSDORead.index := 16#2942;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2942_PcmAcceleration1_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2942_PcmAcceleration1_r);
					48:  (* PCM Acceleration 2 *)
						fbEplSDORead.index := 16#2943;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2943_PcmAcceleration2_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2943_PcmAcceleration2_r);
					49:  (* PCM Acceleration 3 *)
						fbEplSDORead.index := 16#2944;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2944_PcmAcceleration3_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2944_PcmAcceleration3_r);
					50:  (* PCM Acceleration 4 *)
						fbEplSDORead.index := 16#2945;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2945_PcmAcceleration4_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2945_PcmAcceleration4_r);
					51:  (* PCM Acceleration 5 *)
						fbEplSDORead.index := 16#2946;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2946_PcmAcceleration5_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2946_PcmAcceleration5_r);
					52:  (* PCM Acceleration 6 *)
						fbEplSDORead.index := 16#2947;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2947_PcmAcceleration6_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2947_PcmAcceleration6_r);
					53:  (* PCM Acceleration 7 *)
						fbEplSDORead.index := 16#2948;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2948_PcmAcceleration7_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2948_PcmAcceleration7_r);
					54:  (* PCM Velocitiy 1 *)
						fbEplSDORead.index := 16#2962;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2962_PcmVelocity1_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2962_PcmVelocity1_r);
					55:  (* PCM Velocitiy 2 *)
						fbEplSDORead.index := 16#2963;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2963_PcmVelocity2_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2963_PcmVelocity2_r);
					56:  (* PCM Velocitiy 3 *)
						fbEplSDORead.index := 16#2964;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2964_PcmVelocity3_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2964_PcmVelocity3_r);
					57:  (* PCM Velocitiy 4 *)
						fbEplSDORead.index := 16#2965;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2965_PcmVelocity4_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2965_PcmVelocity4_r);
					58:  (* PCM Velocitiy 5 *)
						fbEplSDORead.index := 16#2966;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2966_PcmVelocity5_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2966_PcmVelocity5_r);
					59:  (* PCM Velocitiy 6 *)
						fbEplSDORead.index := 16#2967;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2967_PcmVelocity6_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2967_PcmVelocity6_r);
					60:  (* PCM Velocitiy 7 *)
						fbEplSDORead.index := 16#2968;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2968_PcmVelocity7_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2968_PcmVelocity7_r);
					61:  (* PCM Deceleration 1 *)
						fbEplSDORead.index := 16#2982;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2982_PcmDeceleration1_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2982_PcmDeceleration1_r);
					62:  (* PCM Deceleration 2 *)
						fbEplSDORead.index := 16#2983;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2983_PcmDeceleration2_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2983_PcmDeceleration2_r);
					63:  (* PCM Deceleration 3 *)
						fbEplSDORead.index := 16#2984;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2984_PcmDeceleration3_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2984_PcmDeceleration3_r);
					64:  (* PCM Deceleration 4 *)
						fbEplSDORead.index := 16#2985;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2985_PcmDeceleration4_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2985_PcmDeceleration4_r);
					65:  (* PCM Deceleration 5 *)
						fbEplSDORead.index := 16#2986;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2986_PcmDeceleration5_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2986_PcmDeceleration5_r);
					66:  (* PCM Deceleration 6 *)
						fbEplSDORead.index := 16#2987;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2987_PcmDeceleration6_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2987_PcmDeceleration6_r);
					67:  (* PCM Deceleration 7 *)
						fbEplSDORead.index := 16#2988;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2988_PcmDeceleration7_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2988_PcmDeceleration7_r);
					68:  (* Output Stage Temperature *)
						fbEplSDORead.index := 16#2A01;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A01_OutputStageTemperature_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A01_OutputStageTemperature_r);
					69:  (* Voltage of Control *)
						fbEplSDORead.index := 16#2A02;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A02_VoltageOfControl_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A02_VoltageOfControl_r);
					70:  (* Voltage of Output Stage *)
						fbEplSDORead.index := 16#2A03;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A03_VoltageOfOutputStage_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A03_VoltageOfOutputStage_r);
					71:  (* Voltage of Battery *)
						fbEplSDORead.index := 16#2A04;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A04_VoltageOfBattery_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A04_VoltageOfBattery_r);
					72:  (* Motor Current *)
						fbEplSDORead.index := 16#2A05;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A05_MotorCurrent_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A05_MotorCurrent_r);
					73:  (* Actual Position *)
						fbEplSDORead.index := 16#2A06;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A06_ActualPosition_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A06_ActualPosition_r);
					74:  (* Actual Rotational Speed *)
						fbEplSDORead.index := 16#2A07;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A07_ActualRotationalSpeed_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A07_ActualRotationalSpeed_r);
					75:  (* Serial Number *)
						fbEplSDORead.index := 16#2A08;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A08_SerialNumber_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A08_SerialNumber_r);
					76:  (* Production Date *)
						fbEplSDORead.index := 16#2A09;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A09_ProductionDate_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A09_ProductionDate_r);
					77:  (* SW Motor Controller *)
						fbEplSDORead.index := 16#2A0A;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A0A_SwMotorController_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A0A_SwMotorController_r);
					78:  (* Gear Reduction *)
						fbEplSDORead.index := 16#2A0B;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A0B_GearReduction_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A0B_GearReduction_r);
					79:  (* System Status Word *)
						fbEplSDORead.index := 16#2A0C;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A0C_SystemStatusWord_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A0C_SystemStatusWord_r);
					80:  (* Encoder Resolution *)
						fbEplSDORead.index := 16#2A0D;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A0D_EncoderResolution_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A0D_EncoderResolution_r);
					81:  (* Device ID *)
						fbEplSDORead.index := 16#2A0E;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2A0E_DeviceId_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2A0E_DeviceId_r);
					82:  (* Number of Errors *)
						fbEplSDORead.index := 16#2B01;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B01_NumberOfErrors_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B01_NumberOfErrors_r);
					83:  (* Error Number 1 *)
						fbEplSDORead.index := 16#2B02;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B02_ErrorNumber1_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B02_ErrorNumber1_r);
					84:  (* Error Number 2 *)
						fbEplSDORead.index := 16#2B03;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B03_ErrorNumber2_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B03_ErrorNumber2_r);
					85:  (* Error Number 3 *)
						fbEplSDORead.index := 16#2B04;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B04_ErrorNumber3_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B04_ErrorNumber3_r);
					86:  (* Error Number 4 *)
						fbEplSDORead.index := 16#2B05;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B05_ErrorNumber4_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B05_ErrorNumber4_r);
					87:  (* Error Number 5 *)
						fbEplSDORead.index := 16#2B06;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B06_ErrorNumber5_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B06_ErrorNumber5_r);
					88:  (* Error Number 6 *)
						fbEplSDORead.index := 16#2B07;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B07_ErrorNumber6_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B07_ErrorNumber6_r);
					89:  (* Error Number 7 *)
						fbEplSDORead.index := 16#2B08;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B08_ErrorNumber7_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B08_ErrorNumber7_r);
					90:  (* Error Number 8 *)
						fbEplSDORead.index := 16#2B09;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B09_ErrorNumber8_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B09_ErrorNumber8_r);
					91:  (* Error Number 9 *)
						fbEplSDORead.index := 16#2B0A;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B0A_ErrorNumber9_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B0A_ErrorNumber9_r);
					92:  (* Error Number 10 *)
						fbEplSDORead.index := 16#2B0B;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2B0B_ErrorNumber10_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2B0B_ErrorNumber10_r);
					93:  (* S-Command *)
						fbEplSDORead.index := 16#2C01;
						fbEplSDORead.subindex := 0;
						fbEplSDORead.pData := ADR(ParameterAg2x.n2C01_Scommand_r);
						fbEplSDORead.datalen := SIZEOF(ParameterAg2x.n2C01_Scommand_r);
				END_CASE;
				rdState := RD_STATE_EXEC_EPL_SDO_READ;
			END_IF;	
			counterRead := counter_read;
		RD_STATE_EXEC_EPL_SDO_READ: (* execute EplSDORead *)
			fbEplSDORead(enable := TRUE, node:=nodeId);
			IF fbEplSDORead.status <> 16#FFFF THEN
				IF fbEplSDORead.status = 0 THEN
					(* read success *)
					rdState := RD_STATE_INIT_EPL_SDO_READ;
				ELSE
					(* read error *)
					counterRead := counter_read;
					counter_read := 0;
					rdState := RD_STATE_WAIT_RTRIG;
					done:=FALSE;
					busy:=FALSE;
					error:=TRUE;
					status := fbEplSDORead.status;
					errorinfo := fbEplSDORead.errorinfo;
				END_IF;
			END_IF;
	END_CASE;
	
	CASE wrState OF
		WR_STATE_WAIT_RTRIG: (* wait for rising edge at input startWrite *)
			IF osrStartWrite AND (rdState = 0) AND (copyState = 0) AND (rdDataObjState = 0) AND (wrDataObjState = 0) THEN
				IF (parameterValid = TRUE) THEN
					counter_write := 0;
					counterWrite := 0;
					done:=FALSE;
					busy:=TRUE;
					error:=FALSE;
					status := 16#FFFF; (* busy *)
					errorinfo := 0;
					wrState := WR_STATE_INIT_EPL_SDO_WRITE;
				ELSE
					busy := 0;
					error := 1;
					done := 0;
					status := 50000; (* no data object loaded *)
					errorinfo := 0;
				END_IF;	
			END_IF;	
		WR_STATE_INIT_EPL_SDO_WRITE: (* start fbEplSDOWrite *)
			counter_write := counter_write + 1;
			IF (counter_write > 66) THEN
				(* no more parameters to write *)
				counter_write := 0;
				counterWrite := 0;
				wrState := WR_STATE_WAIT_RTRIG;
				busy := 0;
				error := 0;
				done := 1;
				status := 0; (* no error *)
				errorinfo := 0;
			ELSE
				(* init EplSDOWrite *)
				CASE counter_write OF
					1:  
						IF ParameterAg2x.n2201_LedFunctionality_r <> ParameterAg2x.n2201_LedFunctionality_w
							THEN
							fbEplSDOWrite.index := 16#2201;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2201_LedFunctionality_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2201_LedFunctionality_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					2:  
						IF ParameterAg2x.n2221_ServiceInterfaceBaudrate_r <> ParameterAg2x.n2221_ServiceInterfaceBaudrate_w
							THEN
							fbEplSDOWrite.index := 16#2221;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2221_ServiceInterfaceBaudrate_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2221_ServiceInterfaceBaudrate_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					3:  
						IF ParameterAg2x.n2301_Do1Functionality_r <> ParameterAg2x.n2301_Do1Functionality_w
							THEN
							fbEplSDOWrite.index := 16#2301;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2301_Do1Functionality_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2301_Do1Functionality_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					4:  
						IF ParameterAg2x.n2303_DoPolarity_r <> ParameterAg2x.n2303_DoPolarity_w
							THEN
							fbEplSDOWrite.index := 16#2303;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2303_DoPolarity_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2303_DoPolarity_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					5:  
						IF ParameterAg2x.n2401_Di1Functionality_r <> ParameterAg2x.n2401_Di1Functionality_w
							THEN
							fbEplSDOWrite.index := 16#2401;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2401_Di1Functionality_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2401_Di1Functionality_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					6:  
						IF ParameterAg2x.n2402_Di2Functionality_r <> ParameterAg2x.n2402_Di2Functionality_w
							THEN
							fbEplSDOWrite.index := 16#2402;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2402_Di2Functionality_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2402_Di2Functionality_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					7:  
						IF ParameterAg2x.n2403_Di3Functionality_r <> ParameterAg2x.n2403_Di3Functionality_w
							THEN
							fbEplSDOWrite.index := 16#2403;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2403_Di3Functionality_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2403_Di3Functionality_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					8:  
						IF ParameterAg2x.n2404_Di4Functionality_r <> ParameterAg2x.n2404_Di4Functionality_w
							THEN
							fbEplSDOWrite.index := 16#2404;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2404_Di4Functionality_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2404_Di4Functionality_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					9:  
						IF ParameterAg2x.n2406_DiPolarity_r <> ParameterAg2x.n2406_DiPolarity_w
							THEN
							fbEplSDOWrite.index := 16#2406;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2406_DiPolarity_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2406_DiPolarity_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					10:  
						IF ParameterAg2x.n2601_ControllerParameterP_r <> ParameterAg2x.n2601_ControllerParameterP_w
							THEN
							fbEplSDOWrite.index := 16#2601;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2601_ControllerParameterP_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2601_ControllerParameterP_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					11:  
						IF ParameterAg2x.n2602_ControllerParameterI_r <> ParameterAg2x.n2602_ControllerParameterI_w
							THEN
							fbEplSDOWrite.index := 16#2602;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2602_ControllerParameterI_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2602_ControllerParameterI_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					12:  
						IF ParameterAg2x.n2603_ControllerParameterD_r <> ParameterAg2x.n2603_ControllerParameterD_w
							THEN
							fbEplSDOWrite.index := 16#2603;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2603_ControllerParameterD_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2603_ControllerParameterD_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					13:  
						IF ParameterAg2x.n2604_APos_r <> ParameterAg2x.n2604_APos_w
							THEN
							fbEplSDOWrite.index := 16#2604;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2604_APos_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2604_APos_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					14:  
						IF ParameterAg2x.n2605_VPos_r <> ParameterAg2x.n2605_VPos_w
							THEN
							fbEplSDOWrite.index := 16#2605;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2605_VPos_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2605_VPos_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					15:  
						IF ParameterAg2x.n2606_DPos_r <> ParameterAg2x.n2606_DPos_w
							THEN
							fbEplSDOWrite.index := 16#2606;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2606_DPos_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2606_DPos_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					16:  
						IF ParameterAg2x.n2607_ARot_r <> ParameterAg2x.n2607_ARot_w
							THEN
							fbEplSDOWrite.index := 16#2607;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2607_ARot_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2607_ARot_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					17:  
						IF ParameterAg2x.n2608_AInch_r <> ParameterAg2x.n2608_AInch_w
							THEN
							fbEplSDOWrite.index := 16#2608;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2608_AInch_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2608_AInch_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					18:  
						IF ParameterAg2x.n2609_VInch_r <> ParameterAg2x.n2609_VInch_w
							THEN
							fbEplSDOWrite.index := 16#2609;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2609_VInch_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2609_VInch_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					19:  
						IF ParameterAg2x.n260A_PosWindow_r <> ParameterAg2x.n260A_PosWindow_w
							THEN
							fbEplSDOWrite.index := 16#260A;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n260A_PosWindow_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n260A_PosWindow_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					20:  
						IF ParameterAg2x.n260B_GearRatioNumerator_r <> ParameterAg2x.n260B_GearRatioNumerator_w
							THEN
							fbEplSDOWrite.index := 16#260B;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n260B_GearRatioNumerator_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n260B_GearRatioNumerator_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					21:  
						IF ParameterAg2x.n260C_GearRatioDenominator_r <> ParameterAg2x.n260C_GearRatioDenominator_w
							THEN
							fbEplSDOWrite.index := 16#260C;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n260C_GearRatioDenominator_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n260C_GearRatioDenominator_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					22:  
						IF ParameterAg2x.n260D_SpindlePitch_r <> ParameterAg2x.n260D_SpindlePitch_w
							THEN
							fbEplSDOWrite.index := 16#260D;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n260D_SpindlePitch_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n260D_SpindlePitch_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					23:  
						IF ParameterAg2x.n260E_CalibrationValue_r <> ParameterAg2x.n260E_CalibrationValue_w
							THEN
							fbEplSDOWrite.index := 16#260E;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n260E_CalibrationValue_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n260E_CalibrationValue_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					24:  
						IF ParameterAg2x.n260F_SoftwareLimit1_r <> ParameterAg2x.n260F_SoftwareLimit1_w
							THEN
							fbEplSDOWrite.index := 16#260F;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n260F_SoftwareLimit1_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n260F_SoftwareLimit1_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					25:  
						IF ParameterAg2x.n2610_SoftwareLimit2_r <> ParameterAg2x.n2610_SoftwareLimit2_w
							THEN
							fbEplSDOWrite.index := 16#2610;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2610_SoftwareLimit2_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2610_SoftwareLimit2_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					26:  
						IF ParameterAg2x.n2611_DeltaInch_r <> ParameterAg2x.n2611_DeltaInch_w
							THEN
							fbEplSDOWrite.index := 16#2611;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2611_DeltaInch_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2611_DeltaInch_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					27:  
						IF ParameterAg2x.n2612_SenseOfRotation_r <> ParameterAg2x.n2612_SenseOfRotation_w
							THEN
							fbEplSDOWrite.index := 16#2612;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2612_SenseOfRotation_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2612_SenseOfRotation_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					28:  
						IF ParameterAg2x.n2613_PosType_r <> ParameterAg2x.n2613_PosType_w
							THEN
							fbEplSDOWrite.index := 16#2613;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2613_PosType_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2613_PosType_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					29:  
						IF ParameterAg2x.n2614_OperatingMode_r <> ParameterAg2x.n2614_OperatingMode_w
							THEN
							fbEplSDOWrite.index := 16#2614;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2614_OperatingMode_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2614_OperatingMode_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					30:  
						IF ParameterAg2x.n2615_Inching2StopMode_r <> ParameterAg2x.n2615_Inching2StopMode_w
							THEN
							fbEplSDOWrite.index := 16#2615;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2615_Inching2StopMode_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2615_Inching2StopMode_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					31:  
						IF ParameterAg2x.n2616_InposMode_r <> ParameterAg2x.n2616_InposMode_w
							THEN
							fbEplSDOWrite.index := 16#2616;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2616_InposMode_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2616_InposMode_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					32:  
						IF ParameterAg2x.n2617_LoopLength_r <> ParameterAg2x.n2617_LoopLength_w
							THEN
							fbEplSDOWrite.index := 16#2617;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2617_LoopLength_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2617_LoopLength_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					33:  
						IF ParameterAg2x.n2618_ContouringErrorLimit_r <> ParameterAg2x.n2618_ContouringErrorLimit_w
							THEN
							fbEplSDOWrite.index := 16#2618;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2618_ContouringErrorLimit_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2618_ContouringErrorLimit_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					34:  
						IF ParameterAg2x.n2619_CurrentLimiting_r <> ParameterAg2x.n2619_CurrentLimiting_w
							THEN
							fbEplSDOWrite.index := 16#2619;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2619_CurrentLimiting_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2619_CurrentLimiting_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					35:  
						IF ParameterAg2x.n261A_Inching2Offset_r <> ParameterAg2x.n261A_Inching2Offset_w
							THEN
							fbEplSDOWrite.index := 16#261A;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n261A_Inching2Offset_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n261A_Inching2Offset_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					36:  
						IF ParameterAg2x.n261B_Inching2AccelerationType_r <> ParameterAg2x.n261B_Inching2AccelerationType_w
							THEN
							fbEplSDOWrite.index := 16#261B;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n261B_Inching2AccelerationType_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n261B_Inching2AccelerationType_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					37:  
						IF ParameterAg2x.n261C_OffsetValue_r <> ParameterAg2x.n261C_OffsetValue_w
							THEN
							fbEplSDOWrite.index := 16#261C;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n261C_OffsetValue_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n261C_OffsetValue_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					38:  
						IF ParameterAg2x.n2922_PcmPosition1_r <> ParameterAg2x.n2922_PcmPosition1_w
							THEN
							fbEplSDOWrite.index := 16#2922;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2922_PcmPosition1_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2922_PcmPosition1_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					39:  
						IF ParameterAg2x.n2923_PcmPosition2_r <> ParameterAg2x.n2923_PcmPosition2_w
							THEN
							fbEplSDOWrite.index := 16#2923;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2923_PcmPosition2_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2923_PcmPosition2_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					40:  
						IF ParameterAg2x.n2924_PcmPosition3_r <> ParameterAg2x.n2924_PcmPosition3_w
							THEN
							fbEplSDOWrite.index := 16#2924;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2924_PcmPosition3_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2924_PcmPosition3_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					41:  
						IF ParameterAg2x.n2925_PcmPosition4_r <> ParameterAg2x.n2925_PcmPosition4_w
							THEN
							fbEplSDOWrite.index := 16#2925;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2925_PcmPosition4_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2925_PcmPosition4_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					42:  
						IF ParameterAg2x.n2926_PcmPosition5_r <> ParameterAg2x.n2926_PcmPosition5_w
							THEN
							fbEplSDOWrite.index := 16#2926;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2926_PcmPosition5_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2926_PcmPosition5_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					43:  
						IF ParameterAg2x.n2927_PcmPosition6_r <> ParameterAg2x.n2927_PcmPosition6_w
							THEN
							fbEplSDOWrite.index := 16#2927;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2927_PcmPosition6_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2927_PcmPosition6_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					44:  
						IF ParameterAg2x.n2928_PcmPosition7_r <> ParameterAg2x.n2928_PcmPosition7_w
							THEN
							fbEplSDOWrite.index := 16#2928;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2928_PcmPosition7_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2928_PcmPosition7_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					45:  
						IF ParameterAg2x.n2942_PcmAcceleration1_r <> ParameterAg2x.n2942_PcmAcceleration1_w
							THEN
							fbEplSDOWrite.index := 16#2942;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2942_PcmAcceleration1_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2942_PcmAcceleration1_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					46:  
						IF ParameterAg2x.n2943_PcmAcceleration2_r <> ParameterAg2x.n2943_PcmAcceleration2_w
							THEN
							fbEplSDOWrite.index := 16#2943;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2943_PcmAcceleration2_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2943_PcmAcceleration2_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					47:  
						IF ParameterAg2x.n2944_PcmAcceleration3_r <> ParameterAg2x.n2944_PcmAcceleration3_w
							THEN
							fbEplSDOWrite.index := 16#2944;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2944_PcmAcceleration3_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2944_PcmAcceleration3_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					48:  
						IF ParameterAg2x.n2945_PcmAcceleration4_r <> ParameterAg2x.n2945_PcmAcceleration4_w
							THEN
							fbEplSDOWrite.index := 16#2945;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2945_PcmAcceleration4_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2945_PcmAcceleration4_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					49:  
						IF ParameterAg2x.n2946_PcmAcceleration5_r <> ParameterAg2x.n2946_PcmAcceleration5_w
							THEN
							fbEplSDOWrite.index := 16#2946;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2946_PcmAcceleration5_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2946_PcmAcceleration5_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					50:  
						IF ParameterAg2x.n2947_PcmAcceleration6_r <> ParameterAg2x.n2947_PcmAcceleration6_w
							THEN
							fbEplSDOWrite.index := 16#2947;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2947_PcmAcceleration6_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2947_PcmAcceleration6_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					51:  
						IF ParameterAg2x.n2948_PcmAcceleration7_r <> ParameterAg2x.n2948_PcmAcceleration7_w
							THEN
							fbEplSDOWrite.index := 16#2948;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2948_PcmAcceleration7_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2948_PcmAcceleration7_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					52:  
						IF ParameterAg2x.n2962_PcmVelocity1_r <> ParameterAg2x.n2962_PcmVelocity1_w
							THEN
							fbEplSDOWrite.index := 16#2962;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2962_PcmVelocity1_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2962_PcmVelocity1_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					53:  
						IF ParameterAg2x.n2963_PcmVelocity2_r <> ParameterAg2x.n2963_PcmVelocity2_w
							THEN
							fbEplSDOWrite.index := 16#2963;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2963_PcmVelocity2_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2963_PcmVelocity2_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					54:  
						IF ParameterAg2x.n2964_PcmVelocity3_r <> ParameterAg2x.n2964_PcmVelocity3_w
							THEN
							fbEplSDOWrite.index := 16#2964;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2964_PcmVelocity3_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2964_PcmVelocity3_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					55:  
						IF ParameterAg2x.n2965_PcmVelocity4_r <> ParameterAg2x.n2965_PcmVelocity4_w
							THEN
							fbEplSDOWrite.index := 16#2965;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2965_PcmVelocity4_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2965_PcmVelocity4_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					56:  
						IF ParameterAg2x.n2966_PcmVelocity5_r <> ParameterAg2x.n2966_PcmVelocity5_w
							THEN
							fbEplSDOWrite.index := 16#2966;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2966_PcmVelocity5_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2966_PcmVelocity5_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					57:  
						IF ParameterAg2x.n2967_PcmVelocity6_r <> ParameterAg2x.n2967_PcmVelocity6_w
							THEN
							fbEplSDOWrite.index := 16#2967;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2967_PcmVelocity6_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2967_PcmVelocity6_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					58:  
						IF ParameterAg2x.n2968_PcmVelocity7_r <> ParameterAg2x.n2968_PcmVelocity7_w
							THEN
							fbEplSDOWrite.index := 16#2968;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2968_PcmVelocity7_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2968_PcmVelocity7_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					59:  
						IF ParameterAg2x.n2982_PcmDeceleration1_r <> ParameterAg2x.n2982_PcmDeceleration1_w
							THEN
							fbEplSDOWrite.index := 16#2982;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2982_PcmDeceleration1_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2982_PcmDeceleration1_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					60:  
						IF ParameterAg2x.n2983_PcmDeceleration2_r <> ParameterAg2x.n2983_PcmDeceleration2_w
							THEN
							fbEplSDOWrite.index := 16#2983;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2983_PcmDeceleration2_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2983_PcmDeceleration2_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					61:  
						IF ParameterAg2x.n2984_PcmDeceleration3_r <> ParameterAg2x.n2984_PcmDeceleration3_w
							THEN
							fbEplSDOWrite.index := 16#2984;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2984_PcmDeceleration3_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2984_PcmDeceleration3_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					62:  
						IF ParameterAg2x.n2985_PcmDeceleration4_r <> ParameterAg2x.n2985_PcmDeceleration4_w
							THEN
							fbEplSDOWrite.index := 16#2985;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2985_PcmDeceleration4_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2985_PcmDeceleration4_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					63:  
						IF ParameterAg2x.n2986_PcmDeceleration5_r <> ParameterAg2x.n2986_PcmDeceleration5_w
							THEN
							fbEplSDOWrite.index := 16#2986;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2986_PcmDeceleration5_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2986_PcmDeceleration5_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					64:  
						IF ParameterAg2x.n2987_PcmDeceleration6_r <> ParameterAg2x.n2987_PcmDeceleration6_w
							THEN
							fbEplSDOWrite.index := 16#2987;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2987_PcmDeceleration6_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2987_PcmDeceleration6_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					65:  
						IF ParameterAg2x.n2988_PcmDeceleration7_r <> ParameterAg2x.n2988_PcmDeceleration7_w
							THEN
							fbEplSDOWrite.index := 16#2988;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2988_PcmDeceleration7_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2988_PcmDeceleration7_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
					66:  
						IF ParameterAg2x.n2C01_Scommand_r <> ParameterAg2x.n2C01_Scommand_w
							THEN
							fbEplSDOWrite.index := 16#2C01;
							fbEplSDOWrite.subindex := 0;
							fbEplSDOWrite.pData := ADR(ParameterAg2x.n2C01_Scommand_w);
							fbEplSDOWrite.datalen := SIZEOF(ParameterAg2x.n2C01_Scommand_w);
							wrState := WR_STATE_EXEC_EPL_SDO_WRITE;
						ELSE
							wrState := WR_STATE_INIT_EPL_SDO_WRITE;
						END_IF;
				END_CASE;
			END_IF;
			counterWrite := counter_write;
		WR_STATE_EXEC_EPL_SDO_WRITE: (* execute EplSDOWrite *)
			fbEplSDOWrite(enable:=TRUE, node:= nodeId);
			IF (fbEplSDOWrite.status <> 16#FFFF) THEN
				IF fbEplSDOWrite.status = 0 THEN
					(* write success *)
					wrState := WR_STATE_INIT_EPL_SDO_WRITE;
					CASE counter_write OF
						1:  
							ParameterAg2x.n2201_LedFunctionality_r := ParameterAg2x.n2201_LedFunctionality_w;
						2:  
							ParameterAg2x.n2221_ServiceInterfaceBaudrate_r := ParameterAg2x.n2221_ServiceInterfaceBaudrate_w;
						3:  
							ParameterAg2x.n2301_Do1Functionality_r := ParameterAg2x.n2301_Do1Functionality_w;
						4:  
							ParameterAg2x.n2303_DoPolarity_r := ParameterAg2x.n2303_DoPolarity_w;
						5:  
							ParameterAg2x.n2401_Di1Functionality_r := ParameterAg2x.n2401_Di1Functionality_w;
						6:  
							ParameterAg2x.n2402_Di2Functionality_r := ParameterAg2x.n2402_Di2Functionality_w;
						7:  
							ParameterAg2x.n2403_Di3Functionality_r := ParameterAg2x.n2403_Di3Functionality_w;
						8:  
							ParameterAg2x.n2404_Di4Functionality_r := ParameterAg2x.n2404_Di4Functionality_w;
						9:  
							ParameterAg2x.n2406_DiPolarity_r := ParameterAg2x.n2406_DiPolarity_w;
						10:  
							ParameterAg2x.n2601_ControllerParameterP_r := ParameterAg2x.n2601_ControllerParameterP_w;
						11:  
							ParameterAg2x.n2602_ControllerParameterI_r := ParameterAg2x.n2602_ControllerParameterI_w;
						12:  
							ParameterAg2x.n2603_ControllerParameterD_r := ParameterAg2x.n2603_ControllerParameterD_w;
						13:  
							ParameterAg2x.n2604_APos_r := ParameterAg2x.n2604_APos_w;
						14:  
							ParameterAg2x.n2605_VPos_r := ParameterAg2x.n2605_VPos_w;
						15:  
							ParameterAg2x.n2606_DPos_r := ParameterAg2x.n2606_DPos_w;
						16:  
							ParameterAg2x.n2607_ARot_r := ParameterAg2x.n2607_ARot_w;
						17:  
							ParameterAg2x.n2608_AInch_r := ParameterAg2x.n2608_AInch_w;
						18:  
							ParameterAg2x.n2609_VInch_r := ParameterAg2x.n2609_VInch_w;
						19:  
							ParameterAg2x.n260A_PosWindow_r := ParameterAg2x.n260A_PosWindow_w;
						20:  
							ParameterAg2x.n260B_GearRatioNumerator_r := ParameterAg2x.n260B_GearRatioNumerator_w;
						21:  
							ParameterAg2x.n260C_GearRatioDenominator_r := ParameterAg2x.n260C_GearRatioDenominator_w;
						22:  
							ParameterAg2x.n260D_SpindlePitch_r := ParameterAg2x.n260D_SpindlePitch_w;
						23:  
							ParameterAg2x.n260E_CalibrationValue_r := ParameterAg2x.n260E_CalibrationValue_w;
						24:  
							ParameterAg2x.n260F_SoftwareLimit1_r := ParameterAg2x.n260F_SoftwareLimit1_w;
						25:  
							ParameterAg2x.n2610_SoftwareLimit2_r := ParameterAg2x.n2610_SoftwareLimit2_w;
						26:  
							ParameterAg2x.n2611_DeltaInch_r := ParameterAg2x.n2611_DeltaInch_w;
						27:  
							ParameterAg2x.n2612_SenseOfRotation_r := ParameterAg2x.n2612_SenseOfRotation_w;
						28:  
							ParameterAg2x.n2613_PosType_r := ParameterAg2x.n2613_PosType_w;
						29:  
							ParameterAg2x.n2614_OperatingMode_r := ParameterAg2x.n2614_OperatingMode_w;
						30:  
							ParameterAg2x.n2615_Inching2StopMode_r := ParameterAg2x.n2615_Inching2StopMode_w;
						31:  
							ParameterAg2x.n2616_InposMode_r := ParameterAg2x.n2616_InposMode_w;
						32:  
							ParameterAg2x.n2617_LoopLength_r := ParameterAg2x.n2617_LoopLength_w;
						33:  
							ParameterAg2x.n2618_ContouringErrorLimit_r := ParameterAg2x.n2618_ContouringErrorLimit_w;
						34:  
							ParameterAg2x.n2619_CurrentLimiting_r := ParameterAg2x.n2619_CurrentLimiting_w;
						35:  
							ParameterAg2x.n261A_Inching2Offset_r := ParameterAg2x.n261A_Inching2Offset_w;
						36:  
							ParameterAg2x.n261B_Inching2AccelerationType_r := ParameterAg2x.n261B_Inching2AccelerationType_w;
						37:  
							ParameterAg2x.n261C_OffsetValue_r := ParameterAg2x.n261C_OffsetValue_w;
						38:  
							ParameterAg2x.n2922_PcmPosition1_r := ParameterAg2x.n2922_PcmPosition1_w;
						39:  
							ParameterAg2x.n2923_PcmPosition2_r := ParameterAg2x.n2923_PcmPosition2_w;
						40:  
							ParameterAg2x.n2924_PcmPosition3_r := ParameterAg2x.n2924_PcmPosition3_w;
						41:  
							ParameterAg2x.n2925_PcmPosition4_r := ParameterAg2x.n2925_PcmPosition4_w;
						42:  
							ParameterAg2x.n2926_PcmPosition5_r := ParameterAg2x.n2926_PcmPosition5_w;
						43:  
							ParameterAg2x.n2927_PcmPosition6_r := ParameterAg2x.n2927_PcmPosition6_w;
						44:  
							ParameterAg2x.n2928_PcmPosition7_r := ParameterAg2x.n2928_PcmPosition7_w;
						45:  
							ParameterAg2x.n2942_PcmAcceleration1_r := ParameterAg2x.n2942_PcmAcceleration1_w;
						46:  
							ParameterAg2x.n2943_PcmAcceleration2_r := ParameterAg2x.n2943_PcmAcceleration2_w;
						47:  
							ParameterAg2x.n2944_PcmAcceleration3_r := ParameterAg2x.n2944_PcmAcceleration3_w;
						48:  
							ParameterAg2x.n2945_PcmAcceleration4_r := ParameterAg2x.n2945_PcmAcceleration4_w;
						49:  
							ParameterAg2x.n2946_PcmAcceleration5_r := ParameterAg2x.n2946_PcmAcceleration5_w;
						50:  
							ParameterAg2x.n2947_PcmAcceleration6_r := ParameterAg2x.n2947_PcmAcceleration6_w;
						51:  
							ParameterAg2x.n2948_PcmAcceleration7_r := ParameterAg2x.n2948_PcmAcceleration7_w;
						52:  
							ParameterAg2x.n2962_PcmVelocity1_r := ParameterAg2x.n2962_PcmVelocity1_w;
						53:  
							ParameterAg2x.n2963_PcmVelocity2_r := ParameterAg2x.n2963_PcmVelocity2_w;
						54:  
							ParameterAg2x.n2964_PcmVelocity3_r := ParameterAg2x.n2964_PcmVelocity3_w;
						55:  
							ParameterAg2x.n2965_PcmVelocity4_r := ParameterAg2x.n2965_PcmVelocity4_w;
						56:  
							ParameterAg2x.n2966_PcmVelocity5_r := ParameterAg2x.n2966_PcmVelocity5_w;
						57:  
							ParameterAg2x.n2967_PcmVelocity6_r := ParameterAg2x.n2967_PcmVelocity6_w;
						58:  
							ParameterAg2x.n2968_PcmVelocity7_r := ParameterAg2x.n2968_PcmVelocity7_w;
						59:  
							ParameterAg2x.n2982_PcmDeceleration1_r := ParameterAg2x.n2982_PcmDeceleration1_w;
						60:  
							ParameterAg2x.n2983_PcmDeceleration2_r := ParameterAg2x.n2983_PcmDeceleration2_w;
						61:  
							ParameterAg2x.n2984_PcmDeceleration3_r := ParameterAg2x.n2984_PcmDeceleration3_w;
						62:  
							ParameterAg2x.n2985_PcmDeceleration4_r := ParameterAg2x.n2985_PcmDeceleration4_w;
						63:  
							ParameterAg2x.n2986_PcmDeceleration5_r := ParameterAg2x.n2986_PcmDeceleration5_w;
						64:  
							ParameterAg2x.n2987_PcmDeceleration6_r := ParameterAg2x.n2987_PcmDeceleration6_w;
						65:  
							ParameterAg2x.n2988_PcmDeceleration7_r := ParameterAg2x.n2988_PcmDeceleration7_w;
						66:  
							ParameterAg2x.n2C01_Scommand_r := ParameterAg2x.n2C01_Scommand_w;
					END_CASE;	
				ELSE
					(* write error *)
					counterWrite := counter_write;
					counter_write := 0;
					wrState := 0;
					done:=FALSE;
					busy:=FALSE;
					error:=TRUE;
					status := fbEplSDOWrite.status;
					errorinfo := fbEplSDOWrite.errorinfo;
					fbEplSDORead.enable := 0;
				END_IF;
			END_IF;	
	END_CASE;		
	
	CASE copyState OF
		COPY_STATE_WAIT_RTRIG:
			IF osrStartCopy AND (rdState = 0) AND (wrState = 0) AND (rdDataObjState = 0) AND (wrDataObjState = 0) THEN
				IF (parameterValid = TRUE) THEN	
					counter_copy := 1;
					counterCopy := 1;
					done:=FALSE;
					busy:=TRUE;
					error:=FALSE;
					status := 16#FFFF; (* busy *)
					errorinfo := 0;
					copyState := COPY_STATE_COPY_R_TO_W;
				ELSE
					busy := 0;
					error := 1;
					done := 0;
					status := 50000; (* no data object loaded *)
					errorinfo := 0;
				END_IF;	
			END_IF;	
		COPY_STATE_COPY_R_TO_W:
			counter_copy := counter_copy + 1;
			IF (counter_copy > 65) THEN
				counter_copy := 0;
				counterCopy := 0;
				copyState := COPY_STATE_WAIT_RTRIG;
				busy := 0;
				error := 0;
				done := 1;
				status := 0; (* no error *)
				errorinfo := 0;
			ELSE
				CASE counter_copy OF
					1:  
						ParameterAg2x.n2201_LedFunctionality_w := ParameterAg2x.n2201_LedFunctionality_r;
					2:  
						ParameterAg2x.n2221_ServiceInterfaceBaudrate_w := ParameterAg2x.n2221_ServiceInterfaceBaudrate_r;
					3:  
						ParameterAg2x.n2301_Do1Functionality_w := ParameterAg2x.n2301_Do1Functionality_r;
					4:  
						ParameterAg2x.n2303_DoPolarity_w := ParameterAg2x.n2303_DoPolarity_r;
					5:  
						ParameterAg2x.n2401_Di1Functionality_w := ParameterAg2x.n2401_Di1Functionality_r;
					6:  
						ParameterAg2x.n2402_Di2Functionality_w := ParameterAg2x.n2402_Di2Functionality_r;
					7:  
						ParameterAg2x.n2403_Di3Functionality_w := ParameterAg2x.n2403_Di3Functionality_r;
					8:  
						ParameterAg2x.n2404_Di4Functionality_w := ParameterAg2x.n2404_Di4Functionality_r;
					9:  
						ParameterAg2x.n2406_DiPolarity_w := ParameterAg2x.n2406_DiPolarity_r;
					10:  
						ParameterAg2x.n2601_ControllerParameterP_w := ParameterAg2x.n2601_ControllerParameterP_r;
					11:  
						ParameterAg2x.n2602_ControllerParameterI_w := ParameterAg2x.n2602_ControllerParameterI_r;
					12:  
						ParameterAg2x.n2603_ControllerParameterD_w := ParameterAg2x.n2603_ControllerParameterD_r;
					13:  
						ParameterAg2x.n2604_APos_w := ParameterAg2x.n2604_APos_r;
					14:  
						ParameterAg2x.n2605_VPos_w := ParameterAg2x.n2605_VPos_r;
					15:  
						ParameterAg2x.n2606_DPos_w := ParameterAg2x.n2606_DPos_r;
					16:  
						ParameterAg2x.n2607_ARot_w := ParameterAg2x.n2607_ARot_r;
					17:  
						ParameterAg2x.n2608_AInch_w := ParameterAg2x.n2608_AInch_r;
					18:  
						ParameterAg2x.n2609_VInch_w := ParameterAg2x.n2609_VInch_r;
					19:  
						ParameterAg2x.n260A_PosWindow_w := ParameterAg2x.n260A_PosWindow_r;
					20:  
						ParameterAg2x.n260B_GearRatioNumerator_w := ParameterAg2x.n260B_GearRatioNumerator_r;
					21:  
						ParameterAg2x.n260C_GearRatioDenominator_w := ParameterAg2x.n260C_GearRatioDenominator_r;
					22:  
						ParameterAg2x.n260D_SpindlePitch_w := ParameterAg2x.n260D_SpindlePitch_r;
					23:  
						ParameterAg2x.n260E_CalibrationValue_w := ParameterAg2x.n260E_CalibrationValue_r;
					24:  
						ParameterAg2x.n260F_SoftwareLimit1_w := ParameterAg2x.n260F_SoftwareLimit1_r;
					25:  
						ParameterAg2x.n2610_SoftwareLimit2_w := ParameterAg2x.n2610_SoftwareLimit2_r;
					26:  
						ParameterAg2x.n2611_DeltaInch_w := ParameterAg2x.n2611_DeltaInch_r;
					27:  
						ParameterAg2x.n2612_SenseOfRotation_w := ParameterAg2x.n2612_SenseOfRotation_r;
					28:  
						ParameterAg2x.n2613_PosType_w := ParameterAg2x.n2613_PosType_r;
					29:  
						ParameterAg2x.n2614_OperatingMode_w := ParameterAg2x.n2614_OperatingMode_r;
					30:  
						ParameterAg2x.n2615_Inching2StopMode_w := ParameterAg2x.n2615_Inching2StopMode_r;
					31:  
						ParameterAg2x.n2616_InposMode_w := ParameterAg2x.n2616_InposMode_r;
					32:  
						ParameterAg2x.n2617_LoopLength_w := ParameterAg2x.n2617_LoopLength_r;
					33:  
						ParameterAg2x.n2618_ContouringErrorLimit_w := ParameterAg2x.n2618_ContouringErrorLimit_r;
					34:  
						ParameterAg2x.n2619_CurrentLimiting_w := ParameterAg2x.n2619_CurrentLimiting_r;
					35:  
						ParameterAg2x.n261A_Inching2Offset_w := ParameterAg2x.n261A_Inching2Offset_r;
					36:  
						ParameterAg2x.n261B_Inching2AccelerationType_w := ParameterAg2x.n261B_Inching2AccelerationType_r;
					37:  
						ParameterAg2x.n261C_OffsetValue_w := ParameterAg2x.n261C_OffsetValue_r;
					38:  
						ParameterAg2x.n2922_PcmPosition1_w := ParameterAg2x.n2922_PcmPosition1_r;
					39:  
						ParameterAg2x.n2923_PcmPosition2_w := ParameterAg2x.n2923_PcmPosition2_r;
					40:  
						ParameterAg2x.n2924_PcmPosition3_w := ParameterAg2x.n2924_PcmPosition3_r;
					41:  
						ParameterAg2x.n2925_PcmPosition4_w := ParameterAg2x.n2925_PcmPosition4_r;
					42:  
						ParameterAg2x.n2926_PcmPosition5_w := ParameterAg2x.n2926_PcmPosition5_r;
					43:  
						ParameterAg2x.n2927_PcmPosition6_w := ParameterAg2x.n2927_PcmPosition6_r;
					44:  
						ParameterAg2x.n2928_PcmPosition7_w := ParameterAg2x.n2928_PcmPosition7_r;
					45:  
						ParameterAg2x.n2942_PcmAcceleration1_w := ParameterAg2x.n2942_PcmAcceleration1_r;
					46:  
						ParameterAg2x.n2943_PcmAcceleration2_w := ParameterAg2x.n2943_PcmAcceleration2_r;
					47:  
						ParameterAg2x.n2944_PcmAcceleration3_w := ParameterAg2x.n2944_PcmAcceleration3_r;
					48:  
						ParameterAg2x.n2945_PcmAcceleration4_w := ParameterAg2x.n2945_PcmAcceleration4_r;
					49:  
						ParameterAg2x.n2946_PcmAcceleration5_w := ParameterAg2x.n2946_PcmAcceleration5_r;
					50:  
						ParameterAg2x.n2947_PcmAcceleration6_w := ParameterAg2x.n2947_PcmAcceleration6_r;
					51:  
						ParameterAg2x.n2948_PcmAcceleration7_w := ParameterAg2x.n2948_PcmAcceleration7_r;
					52:  
						ParameterAg2x.n2962_PcmVelocity1_w := ParameterAg2x.n2962_PcmVelocity1_r;
					53:  
						ParameterAg2x.n2963_PcmVelocity2_w := ParameterAg2x.n2963_PcmVelocity2_r;
					54:  
						ParameterAg2x.n2964_PcmVelocity3_w := ParameterAg2x.n2964_PcmVelocity3_r;
					55:  
						ParameterAg2x.n2965_PcmVelocity4_w := ParameterAg2x.n2965_PcmVelocity4_r;
					56:  
						ParameterAg2x.n2966_PcmVelocity5_w := ParameterAg2x.n2966_PcmVelocity5_r;
					57:  
						ParameterAg2x.n2967_PcmVelocity6_w := ParameterAg2x.n2967_PcmVelocity6_r;
					58:  
						ParameterAg2x.n2968_PcmVelocity7_w := ParameterAg2x.n2968_PcmVelocity7_r;
					59:  
						ParameterAg2x.n2982_PcmDeceleration1_w := ParameterAg2x.n2982_PcmDeceleration1_r;
					60:  
						ParameterAg2x.n2983_PcmDeceleration2_w := ParameterAg2x.n2983_PcmDeceleration2_r;
					61:  
						ParameterAg2x.n2984_PcmDeceleration3_w := ParameterAg2x.n2984_PcmDeceleration3_r;
					62:  
						ParameterAg2x.n2985_PcmDeceleration4_w := ParameterAg2x.n2985_PcmDeceleration4_r;
					63:  
						ParameterAg2x.n2986_PcmDeceleration5_w := ParameterAg2x.n2986_PcmDeceleration5_r;
					64:  
						ParameterAg2x.n2987_PcmDeceleration6_w := ParameterAg2x.n2987_PcmDeceleration6_r;
					65:  
						ParameterAg2x.n2988_PcmDeceleration7_w := ParameterAg2x.n2988_PcmDeceleration7_r;
					66:  
						ParameterAg2x.n2C01_Scommand_w := ParameterAg2x.n2C01_Scommand_r;
				END_CASE;
			END_IF;
			counterCopy := counter_copy;
	END_CASE;
	
	CASE rdDataObjState OF
		DORD_STATE_WAIT_RTRIG: (* start condition *)
			IF osrStartReadDataObj AND (rdState = 0) AND (wrState = 0) AND (copyState = 0) AND (wrDataObjState = 0) THEN
				done:=FALSE;
				busy:=TRUE;
				error:=FALSE;
				status := 16#FFFF; (* busy *)
				errorinfo := 0;
				counter_load := 0;
				rdDataObjState := DORD_STATE_IDENTIFY_MODULE;
			END_IF;	
		DORD_STATE_IDENTIFY_MODULE: (* check if DataObj exists *)
			DatObjInfo_0(enable := TRUE, pName := ADR(modulename));
			IF DatObjInfo_0.status = 0 THEN
				(* exists, read *)
				rdDataObjState := DORD_STATE_LOAD_TO_STRUCT;
			ELSIF DatObjInfo_0.status = doERR_MODULNOTFOUND THEN
				(* don't exists, load default values *)
				rdDataObjState := DORD_STATE_LOAD_DEFAULTS;
			ELSE 
				(* error *)
				rdDataObjState := DORD_STATE_WAIT_RTRIG;
				done:=FALSE;
				busy:=FALSE;
				error:=TRUE;
				status := DatObjInfo_0.status;
				errorinfo := 0;
			END_IF
		DORD_STATE_LOAD_DEFAULTS: (* load structure with default values *)
			counter_load := counter_load + 1;
			IF (counter_load > 65) THEN
				counter_load := 0;
				rdDataObjState := DORD_STATE_CREATE_MODULE;
			ELSE
				CASE counter_load OF
					1:  
						ParameterAg2x.n2201_LedFunctionality_w := 0;
					2:  
						ParameterAg2x.n2221_ServiceInterfaceBaudrate_w := 1;
					3:  
						ParameterAg2x.n2301_Do1Functionality_w := 0;
					4:  
						ParameterAg2x.n2303_DoPolarity_w := 0;
					5:  
						ParameterAg2x.n2401_Di1Functionality_w := 0;
					6:  
						ParameterAg2x.n2402_Di2Functionality_w := 0;
					7:  
						ParameterAg2x.n2403_Di3Functionality_w := 0;
					8:  
						ParameterAg2x.n2404_Di4Functionality_w := 0;
					9:  
						ParameterAg2x.n2406_DiPolarity_w := 0;
					10:  
						ParameterAg2x.n2601_ControllerParameterP_w := 300;
					11:  
						ParameterAg2x.n2602_ControllerParameterI_w := 2;
					12:  
						ParameterAg2x.n2603_ControllerParameterD_w := 0;
					13:  
						ParameterAg2x.n2604_APos_w := 50;
					14:  
						ParameterAg2x.n2605_VPos_w := 10;
					15:  
						ParameterAg2x.n2606_DPos_w := 101;
					16:  
						ParameterAg2x.n2607_ARot_w := 50;
					17:  
						ParameterAg2x.n2608_AInch_w := 50;
					18:  
						ParameterAg2x.n2609_VInch_w := 10;
					19:  
						ParameterAg2x.n260A_PosWindow_w := 10;
					20:  
						ParameterAg2x.n260B_GearRatioNumerator_w := 1;
					21:  
						ParameterAg2x.n260C_GearRatioDenominator_w := 1;
					22:  
						ParameterAg2x.n260D_SpindlePitch_w := 0;
					23:  
						ParameterAg2x.n260E_CalibrationValue_w := 0;
					24:  
						ParameterAg2x.n260F_SoftwareLimit1_w := 99999;
					25:  
						ParameterAg2x.n2610_SoftwareLimit2_w := -19999;
					26:  
						ParameterAg2x.n2611_DeltaInch_w := 720;
					27:  
						ParameterAg2x.n2612_SenseOfRotation_w := 0;
					28:  
						ParameterAg2x.n2613_PosType_w := 0;
					29:  
						ParameterAg2x.n2614_OperatingMode_w := 0;
					30:  
						ParameterAg2x.n2615_Inching2StopMode_w := 0;
					31:  
						ParameterAg2x.n2616_InposMode_w := 0;
					32:  
						ParameterAg2x.n2617_LoopLength_w := 360;
					33:  
						ParameterAg2x.n2618_ContouringErrorLimit_w := 400;
					34:  
						ParameterAg2x.n2619_CurrentLimiting_w := 110;
					35:  
						ParameterAg2x.n261A_Inching2Offset_w := 100;
					36:  
						ParameterAg2x.n261B_Inching2AccelerationType_w := 0;
					37:  
						ParameterAg2x.n261C_OffsetValue_w := 0;
					38:  
						ParameterAg2x.n2922_PcmPosition1_w := 0;
					39:  
						ParameterAg2x.n2923_PcmPosition2_w := 0;
					40:  
						ParameterAg2x.n2924_PcmPosition3_w := 0;
					41:  
						ParameterAg2x.n2925_PcmPosition4_w := 0;
					42:  
						ParameterAg2x.n2926_PcmPosition5_w := 0;
					43:  
						ParameterAg2x.n2927_PcmPosition6_w := 0;
					44:  
						ParameterAg2x.n2928_PcmPosition7_w := 0;
					45:  
						ParameterAg2x.n2942_PcmAcceleration1_w := 50;
					46:  
						ParameterAg2x.n2943_PcmAcceleration2_w := 50;
					47:  
						ParameterAg2x.n2944_PcmAcceleration3_w := 50;
					48:  
						ParameterAg2x.n2945_PcmAcceleration4_w := 50;
					49:  
						ParameterAg2x.n2946_PcmAcceleration5_w := 50;
					50:  
						ParameterAg2x.n2947_PcmAcceleration6_w := 50;
					51:  
						ParameterAg2x.n2948_PcmAcceleration7_w := 50;
					52:  
						ParameterAg2x.n2962_PcmVelocity1_w := 10;
					53:  
						ParameterAg2x.n2963_PcmVelocity2_w := 10;
					54:  
						ParameterAg2x.n2964_PcmVelocity3_w := 10;
					55:  
						ParameterAg2x.n2965_PcmVelocity4_w := 10;
					56:  
						ParameterAg2x.n2966_PcmVelocity5_w := 10;
					57:  
						ParameterAg2x.n2967_PcmVelocity6_w := 10;
					58:  
						ParameterAg2x.n2968_PcmVelocity7_w := 10;
					59:  
						ParameterAg2x.n2982_PcmDeceleration1_w := 101;
					60:  
						ParameterAg2x.n2983_PcmDeceleration2_w := 101;
					61:  
						ParameterAg2x.n2984_PcmDeceleration3_w := 101;
					62:  
						ParameterAg2x.n2985_PcmDeceleration4_w := 101;
					63:  
						ParameterAg2x.n2986_PcmDeceleration5_w := 101;
					64:  
						ParameterAg2x.n2987_PcmDeceleration6_w := 101;
					65:  
						ParameterAg2x.n2988_PcmDeceleration7_w := 101;
					66:  
						ParameterAg2x.n2C01_Scommand_w := 0;
				END_CASE;
			END_IF;	
		DORD_STATE_CREATE_MODULE: (* create DataObj *)
			(* DatObjCreate is asynchronous *)
			DatObjCreate_0(enable := TRUE, grp := 0, pName := ADR(modulename), len := SIZEOF(ParameterAg2x), MemType := doUSRROM, Option := 0, pCpyData := ADR(ParameterAg2x));
			IF DatObjCreate_0.status <> 16#FFFF THEN
				IF DatObjCreate_0.status = 0 THEN
					(* executed correctly *)
					rdDataObjState := DORD_STATE_SET_PARAMETER_VALID;
					status := 0;
				ELSE
					(* error *)
					rdDataObjState := DORD_STATE_WAIT_RTRIG;
					done:=FALSE;
					busy:=FALSE;
					error:=TRUE;
					status := DatObjCreate_0.status;
					errorinfo := 0;
				END_IF
			END_IF
		DORD_STATE_LOAD_TO_STRUCT: (* load DataObj to structure *)
			DatObjRead_0(enable := TRUE, ident := DatObjInfo_0.ident, Offset := 0, pDestination := ADR(ParameterAg2x), len := DatObjInfo_0.len);
			IF DatObjRead_0.status = 0 THEN
				(* ready, set parameterValid TRUE *)
				status := 0;
				rdDataObjState := DORD_STATE_SET_PARAMETER_VALID;
			ELSE
				(* error *)
				rdDataObjState := DORD_STATE_WAIT_RTRIG;
				done:=FALSE;
				busy:=FALSE;
				error:=TRUE;
				status := DatObjRead_0.status;
				errorinfo := 0;
			END_IF 
		DORD_STATE_SET_PARAMETER_VALID: (* set parameterValid true *)
			parameterValid := TRUE;
			done:=TRUE;
			busy:=FALSE;
			error:=FALSE;
			status := 0; (* no error *)
			errorinfo := 0;	
			rdDataObjState := DORD_STATE_WAIT_RTRIG;
	END_CASE;	
	
	CASE wrDataObjState OF
		DOWR_STATE_WAIT_RTRIG: (* start condition *)
			IF osrStartWriteDataObj AND (rdState = 0) AND (wrState = 0) AND (copyState = 0) AND (rdDataObjState = 0) THEN
				done:=FALSE;
				busy:=TRUE;
				error:=FALSE;
				status := 16#FFFF; (* busy *)
				errorinfo := 0;
				counter_load := 0;
				wrDataObjState := DOWR_STATE_IDENTIFY_MODULE;
			END_IF;
		DOWR_STATE_IDENTIFY_MODULE:
			DatObjInfo_0(enable := TRUE, pName := ADR(modulename));
			IF DatObjInfo_0.status = 0 THEN
				(* exists, delete *)
				wrDataObjState := DOWR_STATE_DELETE_MODULE;
			ELSIF DatObjInfo_0.status = doERR_MODULNOTFOUND THEN
				(* don't exists, generate *)
				wrDataObjState := DOWR_STATE_CREATE_MODULE;
			ELSE 
				(* error *)
				wrDataObjState := DOWR_STATE_WAIT_RTRIG;
				done:=FALSE;
				busy:=FALSE;
				error:=TRUE;
				status := DatObjInfo_0.status;
				errorinfo := 0;
			END_IF
		DOWR_STATE_DELETE_MODULE:
			DatObjDelete_0.enable := TRUE;
			DatObjDelete_0.ident := DatObjInfo_0.ident;
			DatObjDelete_0();
			IF DatObjDelete_0.status <> 16#FFFF THEN
				IF DatObjDelete_0.status = 0 THEN
					(* ready *)
					wrDataObjState:= DOWR_STATE_CREATE_MODULE;
				ELSE
					(* error *)
					wrDataObjState := DOWR_STATE_WAIT_RTRIG;
					done:=FALSE;
					busy:=FALSE;
					error:=TRUE;
					status := DatObjDelete_0.status;
					errorinfo := 0;
				END_IF
			END_IF
		DOWR_STATE_CREATE_MODULE:
			DatObjCreate_0(enable := TRUE, grp := 0, pName := ADR(modulename), len := SIZEOF(ParameterAg2x), MemType := doUSRROM, Option := 0, pCpyData := ADR(ParameterAg2x));
			IF DatObjCreate_0.status <> 16#FFFF THEN
				wrDataObjState:= DOWR_STATE_WAIT_RTRIG;
				IF DatObjCreate_0.status = 0 THEN
					(* ready *)
					done:=TRUE;
					busy:=FALSE;
					error:=FALSE;
					status := 0; (* no error *)
					errorinfo := 0;
				ELSE
					(* error *)
					done:=FALSE;
					busy:=FALSE;
					error:=TRUE;
					status := DatObjCreate_0.status;
					errorinfo := 0;
				END_IF
			END_IF
	END_CASE;	
	
		(* Status Word *)
	PdInAg2x.StatusWordAg2x.bSupply:= statusWord.0;
	PdInAg2x.StatusWordAg2x.bReadinessToTravel:= statusWord.1;
	PdInAg2x.StatusWordAg2x.bUpperLimit:= statusWord.2;
	PdInAg2x.StatusWordAg2x.bLowerLimit:= statusWord.3;
	PdInAg2x.StatusWordAg2x.bActuatorTravelsStands:= statusWord.4;
	PdInAg2x.StatusWordAg2x.bInPos:= statusWord.5;
	PdInAg2x.StatusWordAg2x.bActiveTravelJob:= statusWord.6;
	PdInAg2x.StatusWordAg2x.bError:= statusWord.7;
	PdInAg2x.StatusWordAg2x.bOperationEnabled:= statusWord.8;
	PdInAg2x.StatusWordAg2x.bSwitchLock:= statusWord.9;
	PdInAg2x.StatusWordAg2x.bTravelJobAcknowledge:= statusWord.10;
	PdInAg2x.StatusWordAg2x.bBatteryWarning:= statusWord.11;
	PdInAg2x.StatusWordAg2x.bCurrentLimiting:= statusWord.12;
	PdInAg2x.StatusWordAg2x.bit13:= statusWord.13;
	PdInAg2x.StatusWordAg2x.bit14:= statusWord.14;
	PdInAg2x.StatusWordAg2x.bit15:= statusWord.15;
	
	(* Actual Value *)
	PdInAg2x.actualValue:= actualValue;
	
	(* Digital Inputs Status *)
	PdInAg2x.DiAg2x.di1:= digitalInputsStatus.0;
	PdInAg2x.DiAg2x.di2:= digitalInputsStatus.1;
	PdInAg2x.DiAg2x.di3:= digitalInputsStatus.2;
	PdInAg2x.DiAg2x.di4:= digitalInputsStatus.3;
	PdInAg2x.DiAg2x.bit4:= digitalInputsStatus.4;
	PdInAg2x.DiAg2x.bit5:= digitalInputsStatus.5;
	PdInAg2x.DiAg2x.bit6:= digitalInputsStatus.6;
	PdInAg2x.DiAg2x.bit7:= digitalInputsStatus.7;

END_FUNCTION_BLOCK